from __future__ import annotations

import operator

from datetime import date
from datetime import datetime
from datetime import timedelta
from typing import TYPE_CHECKING
from typing import Generic
from typing import TypeVar
from typing import cast
from typing import overload

import pendulum

from pendulum.constants import MONTHS_PER_YEAR
from pendulum.duration import Duration
from pendulum.helpers import precise_diff


if TYPE_CHECKING:
    from collections.abc import Iterator

    from typing_extensions import Self
    from typing_extensions import SupportsIndex

    from pendulum.helpers import PreciseDiff
    from pendulum.locales.locale import Locale


_T = TypeVar("_T", bound=date)


class Interval(Duration, Generic[_T]):
    """
    An interval of time between two datetimes.
    """

    def __new__(cls, start: _T, end: _T, absolute: bool = False) -> Self:
        if (isinstance(start, datetime) and not isinstance(end, datetime)) or (
            not isinstance(start, datetime) and isinstance(end, datetime)
        ):
            raise ValueError(
                "Both start and end of an Interval must have the same type"
            )

        if (
            isinstance(start, datetime)
            and isinstance(end, datetime)
            and (
                (start.tzinfo is None and end.tzinfo is not None)
                or (start.tzinfo is not None and end.tzinfo is None)
            )
        ):
            raise TypeError("can't compare offset-naive and offset-aware datetimes")

        if absolute and start > end:
            end, start = start, end

        _start = start
        _end = end
        if isinstance(start, pendulum.DateTime):
            _start = cast(
                "_T",
                datetime(
                    start.year,
                    start.month,
                    start.day,
                    start.hour,
                    start.minute,
                    start.second,
                    start.microsecond,
                    tzinfo=start.tzinfo,
                    fold=start.fold,
                ),
            )
        elif isinstance(start, pendulum.Date):
            _start = cast("_T", date(start.year, start.month, start.day))

        if isinstance(end, pendulum.DateTime):
            _end = cast(
                "_T",
                datetime(
                    end.year,
                    end.month,
                    end.day,
                    end.hour,
                    end.minute,
                    end.second,
                    end.microsecond,
                    tzinfo=end.tzinfo,
                    fold=end.fold,
                ),
            )
        elif isinstance(end, pendulum.Date):
            _end = cast("_T", date(end.year, end.month, end.day))

        # Fixing issues with datetime.__sub__()
        # not handling offsets if the tzinfo is the same
        if (
            isinstance(_start, datetime)
            and isinstance(_end, datetime)
            and _start.tzinfo is _end.tzinfo
        ):
            if _start.tzinfo is not None:
                offset = cast("timedelta", cast("datetime", start).utcoffset())
                _start = cast("_T", (_start - offset).replace(tzinfo=None))

            if isinstance(end, datetime) and _end.tzinfo is not None:
                offset = cast("timedelta", end.utcoffset())
                _end = cast("_T", (_end - offset).replace(tzinfo=None))

        delta: timedelta = _end - _start

        return super().__new__(cls, seconds=delta.total_seconds())

    def __init__(self, start: _T, end: _T, absolute: bool = False) -> None:
        super().__init__()

        _start: _T
        if not isinstance(start, pendulum.Date):
            if isinstance(start, datetime):
                start = cast("_T", pendulum.instance(start))
            else:
                start = cast("_T", pendulum.date(start.year, start.month, start.day))

            _start = start
        else:
            if isinstance(start, pendulum.DateTime):
                _start = cast(
                    "_T",
                    datetime(
                        start.year,
                        start.month,
                        start.day,
                        start.hour,
                        start.minute,
                        start.second,
                        start.microsecond,
                        tzinfo=start.tzinfo,
                    ),
                )
            else:
                _start = cast("_T", date(start.year, start.month, start.day))

        _end: _T
        if not isinstance(end, pendulum.Date):
            if isinstance(end, datetime):
                end = cast("_T", pendulum.instance(end))
            else:
                end = cast("_T", pendulum.date(end.year, end.month, end.day))

            _end = end
        else:
            if isinstance(end, pendulum.DateTime):
                _end = cast(
                    "_T",
                    datetime(
                        end.year,
                        end.month,
                        end.day,
                        end.hour,
                        end.minute,
                        end.second,
                        end.microsecond,
                        tzinfo=end.tzinfo,
                    ),
                )
            else:
                _end = cast("_T", date(end.year, end.month, end.day))

        self._invert = False
        if start > end:
            self._invert = True

            if absolute:
                end, start = start, end
                _end, _start = _start, _end

        self._absolute = absolute
        self._start: _T = start
        self._end: _T = end
        self._delta: PreciseDiff = precise_diff(_start, _end)

    @property
    def years(self) -> int:
        return self._delta.years

    @property
    def months(self) -> int:
        return self._delta.months

    @property
    def weeks(self) -> int:
        return abs(self._delta.days) // 7 * self._sign(self._delta.days)

    @property
    def days(self) -> int:
        return self._days

    @property
    def remaining_days(self) -> int:
        return abs(self._delta.days) % 7 * self._sign(self._days)

    @property
    def hours(self) -> int:
        return self._delta.hours

    @property
    def minutes(self) -> int:
        return self._delta.minutes

    @property
    def start(self) -> _T:
        return self._start

    @property
    def end(self) -> _T:
        return self._end

    def in_years(self) -> int:
        """
        Gives the duration of the Interval in full years.
        """
        return self.years

    def in_months(self) -> int:
        """
        Gives the duration of the Interval in full months.
        """
        return self.years * MONTHS_PER_YEAR + self.months

    def in_weeks(self) -> int:
        days = self.in_days()
        sign = 1

        if days < 0:
            sign = -1

        return sign * (abs(days) // 7)

    def in_days(self) -> int:
        return self._delta.total_days

    def in_words(self, locale: str | None = None, separator: str = " ") -> str:
        """
        Get the current interval in words in the current locale.

        Ex: 6 jours 23 heures 58 minutes

        :param locale: The locale to use. Defaults to current locale.
        :param separator: The separator to use between each unit
        """
        from pendulum.locales.locale import Locale

        intervals = [
            ("year", self.years),
            ("month", self.months),
            ("week", self.weeks),
            ("day", self.remaining_days),
            ("hour", self.hours),
            ("minute", self.minutes),
            ("second", self.remaining_seconds),
        ]
        loaded_locale: Locale = Locale.load(locale or pendulum.get_locale())
        parts = []
        for interval in intervals:
            unit, interval_count = interval
            if abs(interval_count) > 0:
                translation = loaded_locale.translation(
                    f"units.{unit}.{loaded_locale.plural(abs(interval_count))}"
                )
                parts.append(translation.format(interval_count))

        if not parts:
            count: str | int = 0
            if abs(self.microseconds) > 0:
                unit = f"units.second.{loaded_locale.plural(1)}"
                count = f"{abs(self.microseconds) / 1e6:.2f}"
            else:
                unit = f"units.microsecond.{loaded_locale.plural(0)}"

            translation = loaded_locale.translation(unit)
            parts.append(translation.format(count))

        return separator.join(parts)

    def range(self, unit: str, amount: int = 1) -> Iterator[_T]:
        method = "add"
        op = operator.le
        if not self._absolute and self.invert:
            method = "subtract"
            op = operator.ge

        start, end = self.start, self.end

        i = amount
        while op(start, end):
            yield start

            start = getattr(self.start, method)(**{unit: i})

            i += amount

    def as_duration(self) -> Duration:
        """
        Return the Interval as a Duration.
        """
        return Duration(seconds=self.total_seconds())

    def __iter__(self) -> Iterator[_T]:
        return self.range("days")

    def __contains__(self, item: _T) -> bool:
        return self.start <= item <= self.end

    def __add__(self, other: timedelta) -> Duration:  # type: ignore[override]
        return self.as_duration().__add__(other)

    __radd__ = __add__  # type: ignore[assignment]

    def __sub__(self, other: timedelta) -> Duration:  # type: ignore[override]
        return self.as_duration().__sub__(other)

    def __neg__(self) -> Self:
        return self.__class__(self.end, self.start, self._absolute)

    def __mul__(self, other: int | float) -> Duration:  # type: ignore[override]
        return self.as_duration().__mul__(other)

    __rmul__ = __mul__  # type: ignore[assignment]

    @overload  # type: ignore[override]
    def __floordiv__(self, other: timedelta) -> int: ...

    @overload
    def __floordiv__(self, other: int) -> Duration: ...

    def __floordiv__(self, other: int | timedelta) -> int | Duration:
        return self.as_duration().__floordiv__(other)

    __div__ = __floordiv__  # type: ignore[assignment]

    @overload  # type: ignore[override]
    def __truediv__(self, other: timedelta) -> float: ...

    @overload
    def __truediv__(self, other: float) -> Duration: ...

    def __truediv__(self, other: float | timedelta) -> Duration | float:
        return self.as_duration().__truediv__(other)

    def __mod__(self, other: timedelta) -> Duration:  # type: ignore[override]
        return self.as_duration().__mod__(other)

    def __divmod__(self, other: timedelta) -> tuple[int, Duration]:
        return self.as_duration().__divmod__(other)

    def __abs__(self) -> Self:
        return self.__class__(self.start, self.end, absolute=True)

    def __repr__(self) -> str:
        return f"<Interval [{self._start} -> {self._end}]>"

    def __str__(self) -> str:
        return self.__repr__()

    def _cmp(self, other: timedelta) -> int:
        # Only needed for PyPy
        assert isinstance(other, timedelta)

        if isinstance(other, Interval):
            other = other.as_timedelta()

        td = self.as_timedelta()

        return 0 if td == other else 1 if td > other else -1

    def _getstate(self, protocol: SupportsIndex = 3) -> tuple[_T, _T, bool]:
        start, end = self.start, self.end

        if self._invert and self._absolute:
            end, start = start, end

        return start, end, self._absolute

    def __reduce__(
        self,
    ) -> tuple[type[Self], tuple[_T, _T, bool]]:
        return self.__reduce_ex__(2)

    def __reduce_ex__(
        self, protocol: SupportsIndex
    ) -> tuple[type[Self], tuple[_T, _T, bool]]:
        return self.__class__, self._getstate(protocol)

    def __hash__(self) -> int:
        return hash((self.start, self.end, self._absolute))

    def __eq__(self, other: object) -> bool:
        if isinstance(other, Interval):
            return (self.start, self.end, self._absolute) == (
                other.start,
                other.end,
                other._absolute,
            )
        else:
            return self.as_duration() == other

    def __ne__(self, other: object) -> bool:
        return not self.__eq__(other)



from __future__ import annotations

from pendulum.locales.sk.custom import translations as custom_translations


"""
sk locale file.

It has been generated automatically and must not be modified directly.
"""


locale = {
    "plural": lambda n: "few"
    if ((n == n and (n >= 2 and n <= 4)) and (0 == 0 and (0 == 0)))
    else "many"
    if (not (0 == 0 and (0 == 0)))
    else "one"
    if ((n == n and (n == 1)) and (0 == 0 and (0 == 0)))
    else "other",
    "ordinal": lambda n: "other",
    "translations": {
        "days": {
            "abbreviated": {
                0: "po",
                1: "ut",
                2: "st",
                3: "št",
                4: "pi",
                5: "so",
                6: "ne",
            },
            "narrow": {
                0: "p",
                1: "u",
                2: "s",
                3: "š",
                4: "p",
                5: "s",
                6: "n",
            },
            "short": {
                0: "po",
                1: "ut",
                2: "st",
                3: "št",
                4: "pi",
                5: "so",
                6: "ne",
            },
            "wide": {
                0: "pondelok",
                1: "utorok",
                2: "streda",
                3: "štvrtok",
                4: "piatok",
                5: "sobota",
                6: "nedeľa",
            },
        },
        "months": {
            "abbreviated": {
                1: "jan",
                2: "feb",
                3: "mar",
                4: "apr",
                5: "máj",
                6: "jún",
                7: "júl",
                8: "aug",
                9: "sep",
                10: "okt",
                11: "nov",
                12: "dec",
            },
            "narrow": {
                1: "j",
                2: "f",
                3: "m",
                4: "a",
                5: "m",
                6: "j",
                7: "j",
                8: "a",
                9: "s",
                10: "o",
                11: "n",
                12: "d",
            },
            "wide": {
                1: "januára",
                2: "februára",
                3: "marca",
                4: "apríla",
                5: "mája",
                6: "júna",
                7: "júla",
                8: "augusta",
                9: "septembra",
                10: "októbra",
                11: "novembra",
                12: "decembra",
            },
        },
        "units": {
            "year": {
                "one": "{0} rok",
                "few": "{0} roky",
                "many": "{0} roka",
                "other": "{0} rokov",
            },
            "month": {
                "one": "{0} mesiac",
                "few": "{0} mesiace",
                "many": "{0} mesiaca",
                "other": "{0} mesiacov",
            },
            "week": {
                "one": "{0} týždeň",
                "few": "{0} týždne",
                "many": "{0} týždňa",
                "other": "{0} týždňov",
            },
            "day": {
                "one": "{0} deň",
                "few": "{0} dni",
                "many": "{0} dňa",
                "other": "{0} dní",
            },
            "hour": {
                "one": "{0} hodina",
                "few": "{0} hodiny",
                "many": "{0} hodiny",
                "other": "{0} hodín",
            },
            "minute": {
                "one": "{0} minúta",
                "few": "{0} minúty",
                "many": "{0} minúty",
                "other": "{0} minút",
            },
            "second": {
                "one": "{0} sekunda",
                "few": "{0} sekundy",
                "many": "{0} sekundy",
                "other": "{0} sekúnd",
            },
            "microsecond": {
                "one": "{0} mikrosekunda",
                "few": "{0} mikrosekundy",
                "many": "{0} mikrosekundy",
                "other": "{0} mikrosekúnd",
            },
        },
        "relative": {
            "year": {
                "future": {
                    "other": "o {0} rokov",
                    "one": "o {0} rok",
                    "few": "o {0} roky",
                    "many": "o {0} roka",
                },
                "past": {
                    "other": "pred {0} rokmi",
                    "one": "pred {0} rokom",
                    "few": "pred {0} rokmi",
                    "many": "pred {0} roka",
                },
            },
            "month": {
                "future": {
                    "other": "o {0} mesiacov",
                    "one": "o {0} mesiac",
                    "few": "o {0} mesiace",
                    "many": "o {0} mesiaca",
                },
                "past": {
                    "other": "pred {0} mesiacmi",
                    "one": "pred {0} mesiacom",
                    "few": "pred {0} mesiacmi",
                    "many": "pred {0} mesiaca",
                },
            },
            "week": {
                "future": {
                    "other": "o {0} týždňov",
                    "one": "o {0} týždeň",
                    "few": "o {0} týždne",
                    "many": "o {0} týždňa",
                },
                "past": {
                    "other": "pred {0} týždňami",
                    "one": "pred {0} týždňom",
                    "few": "pred {0} týždňami",
                    "many": "pred {0} týždňa",
                },
            },
            "day": {
                "future": {
                    "other": "o {0} dní",
                    "one": "o {0} deň",
                    "few": "o {0} dni",
                    "many": "o {0} dňa",
                },
                "past": {
                    "other": "pred {0} dňami",
                    "one": "pred {0} dňom",
                    "few": "pred {0} dňami",
                    "many": "pred {0} dňa",
                },
            },
            "hour": {
                "future": {
                    "other": "o {0} hodín",
                    "one": "o {0} hodinu",
                    "few": "o {0} hodiny",
                    "many": "o {0} hodiny",
                },
                "past": {
                    "other": "pred {0} hodinami",
                    "one": "pred {0} hodinou",
                    "few": "pred {0} hodinami",
                    "many": "pred {0} hodinou",
                },
            },
            "minute": {
                "future": {
                    "other": "o {0} minút",
                    "one": "o {0} minútu",
                    "few": "o {0} minúty",
                    "many": "o {0} minúty",
                },
                "past": {
                    "other": "pred {0} minútami",
                    "one": "pred {0} minútou",
                    "few": "pred {0} minútami",
                    "many": "pred {0} minúty",
                },
            },
            "second": {
                "future": {
                    "other": "o {0} sekúnd",
                    "one": "o {0} sekundu",
                    "few": "o {0} sekundy",
                    "many": "o {0} sekundy",
                },
                "past": {
                    "other": "pred {0} sekundami",
                    "one": "pred {0} sekundou",
                    "few": "pred {0} sekundami",
                    "many": "pred {0} sekundy",
                },
            },
        },
        "day_periods": {
            "midnight": "o polnoci",
            "am": "AM",
            "noon": "napoludnie",
            "pm": "PM",
            "morning1": "ráno",
            "morning2": "dopoludnia",
            "afternoon1": "popoludní",
            "evening1": "večer",
            "night1": "v noci",
        },
        "week_data": {
            "min_days": 1,
            "first_day": 0,
            "weekend_start": 5,
            "weekend_end": 6,
        },
    },
    "custom": custom_translations,
}



from __future__ import annotations

from datetime import datetime

import pytz

import pendulum

from tests.conftest import assert_datetime


def test_equal_to_true():
    d1 = pendulum.datetime(2000, 1, 1, 1, 2, 3)
    d2 = pendulum.datetime(2000, 1, 1, 1, 2, 3)
    d3 = datetime(2000, 1, 1, 1, 2, 3, tzinfo=pendulum.UTC)

    assert d2 == d1
    assert d3 == d1


def test_equal_to_false():
    d1 = pendulum.datetime(2000, 1, 1, 1, 2, 3)
    d2 = pendulum.datetime(2000, 1, 2, 1, 2, 3)
    d3 = datetime(2000, 1, 2, 1, 2, 3, tzinfo=pendulum.UTC)

    assert d2 != d1
    assert d3 != d1


def test_equal_with_timezone_true():
    d1 = pendulum.datetime(2000, 1, 1, 12, 0, 0, tz="America/Toronto")
    d2 = pendulum.datetime(2000, 1, 1, 9, 0, 0, tz="America/Vancouver")
    d3 = datetime(2000, 1, 1, 12, 0, 0, tzinfo=pendulum.timezone("America/Toronto"))

    assert d2 == d1
    assert d3 == d1


def test_equal_with_timezone_false():
    d1 = pendulum.datetime(2000, 1, 1, tz="America/Toronto")
    d2 = pendulum.datetime(2000, 1, 1, tz="America/Vancouver")
    d3 = datetime(2000, 1, 1, tzinfo=pendulum.timezone("America/Toronto"))

    assert d2 != d1
    assert d3 == d1


def test_not_equal_to_true():
    d1 = pendulum.datetime(2000, 1, 1, 1, 2, 3)
    d2 = pendulum.datetime(2000, 1, 2, 1, 2, 3)
    d3 = datetime(2000, 1, 2, 1, 2, 3, tzinfo=pendulum.UTC)

    assert d2 != d1
    assert d3 != d1


def test_not_equal_to_false():
    d1 = pendulum.datetime(2000, 1, 1, 1, 2, 3)
    d2 = pendulum.datetime(2000, 1, 1, 1, 2, 3)
    d3 = datetime(2000, 1, 1, 1, 2, 3, tzinfo=pendulum.UTC)

    assert d2 == d1
    assert d3 == d1


def test_not_equal_with_timezone_true():
    d1 = pendulum.datetime(2000, 1, 1, tz="America/Toronto")
    d2 = pendulum.datetime(2000, 1, 1, tz="America/Vancouver")
    d3 = datetime(2000, 1, 1, tzinfo=pendulum.timezone("America/Toronto"))

    assert d2 != d1
    assert d3 == d1


def test_not_equal_to_none():
    d1 = pendulum.datetime(2000, 1, 1, 1, 2, 3)

    assert d1 is not None


def test_greater_than_true():
    d1 = pendulum.datetime(2000, 1, 1)
    d2 = pendulum.datetime(1999, 12, 31)
    d3 = datetime(1999, 12, 31, tzinfo=pendulum.UTC)

    assert d1 > d2
    assert d1 > d3


def test_greater_than_false():
    d1 = pendulum.datetime(2000, 1, 1)
    d2 = pendulum.datetime(2000, 1, 2)
    d3 = datetime(2000, 1, 2, tzinfo=pendulum.UTC)

    assert not d1 > d2
    assert not d1 > d3


def test_greater_than_with_timezone_true():
    d1 = pendulum.datetime(2000, 1, 1, 12, 0, 0, tz="America/Toronto")
    d2 = pendulum.datetime(2000, 1, 1, 8, 59, 59, tz="America/Vancouver")
    d3 = pytz.timezone("America/Vancouver").localize(datetime(2000, 1, 1, 8, 59, 59))

    assert d1 > d2
    assert d1 > d3


def test_greater_than_with_timezone_false():
    d1 = pendulum.datetime(2000, 1, 1, 12, 0, 0, tz="America/Toronto")
    d2 = pendulum.datetime(2000, 1, 1, 9, 0, 1, tz="America/Vancouver")
    d3 = pytz.timezone("America/Vancouver").localize(datetime(2000, 1, 1, 9, 0, 1))

    assert not d1 > d2
    assert not d1 > d3


def test_greater_than_or_equal_true():
    d1 = pendulum.datetime(2000, 1, 1)
    d2 = pendulum.datetime(1999, 12, 31)
    d3 = datetime(1999, 12, 31, tzinfo=pendulum.UTC)

    assert d1 >= d2
    assert d1 >= d3


def test_greater_than_or_equal_true_equal():
    d1 = pendulum.datetime(2000, 1, 1)
    d2 = pendulum.datetime(2000, 1, 1)
    d3 = datetime(2000, 1, 1, tzinfo=pendulum.UTC)

    assert d1 >= d2
    assert d1 >= d3


def test_greater_than_or_equal_false():
    d1 = pendulum.datetime(2000, 1, 1)
    d2 = pendulum.datetime(2000, 1, 2)
    d3 = datetime(2000, 1, 2, tzinfo=pendulum.UTC)

    assert not d1 >= d2
    assert not d1 >= d3


def test_greater_than_or_equal_with_timezone_true():
    d1 = pendulum.datetime(2000, 1, 1, 12, 0, 0, tz="America/Toronto")
    d2 = pendulum.datetime(2000, 1, 1, 8, 59, 59, tz="America/Vancouver")
    d3 = pytz.timezone("America/Vancouver").localize(datetime(2000, 1, 1, 8, 59, 59))

    assert d1 >= d2
    assert d1 >= d3


def test_greater_than_or_equal_with_timezone_false():
    d1 = pendulum.datetime(2000, 1, 1, 12, 0, 0, tz="America/Toronto")
    d2 = pendulum.datetime(2000, 1, 1, 9, 0, 1, tz="America/Vancouver")
    d3 = pytz.timezone("America/Vancouver").localize(datetime(2000, 1, 1, 9, 0, 1))

    assert not d1 >= d2
    assert not d1 >= d3


def test_less_than_true():
    d1 = pendulum.datetime(2000, 1, 1)
    d2 = pendulum.datetime(2000, 1, 2)
    d3 = datetime(2000, 1, 2, tzinfo=pendulum.UTC)

    assert d1 < d2
    assert d1 < d3


def test_less_than_false():
    d1 = pendulum.datetime(2000, 1, 2)
    d2 = pendulum.datetime(2000, 1, 1)
    d3 = datetime(2000, 1, 1, tzinfo=pendulum.UTC)

    assert not d1 < d2
    assert not d1 < d3


def test_less_than_with_timezone_true():
    d1 = pendulum.datetime(2000, 1, 1, 8, 59, 59, tz="America/Vancouver")
    d2 = pendulum.datetime(2000, 1, 1, 12, 0, 0, tz="America/Toronto")
    d3 = pytz.timezone("America/Toronto").localize(datetime(2000, 1, 1, 12, 0, 0))

    assert d1 < d2
    assert d1 < d3


def test_less_than_with_timezone_false():
    d1 = pendulum.datetime(2000, 1, 1, 9, 0, 1, tz="America/Vancouver")
    d2 = pendulum.datetime(2000, 1, 1, 12, 0, 0, tz="America/Toronto")
    d3 = pytz.timezone("America/Toronto").localize(datetime(2000, 1, 1, 12, 0, 0))

    assert not d1 < d2
    assert not d1 < d3


def test_less_than_or_equal_true():
    d1 = pendulum.datetime(2000, 1, 1)
    d2 = pendulum.datetime(2000, 1, 2)
    d3 = datetime(2000, 1, 2, tzinfo=pendulum.UTC)

    assert d1 <= d2
    assert d1 <= d3


def test_less_than_or_equal_true_equal():
    d1 = pendulum.datetime(2000, 1, 1)
    d2 = pendulum.datetime(2000, 1, 1)
    d3 = datetime(2000, 1, 1, tzinfo=pendulum.UTC)

    assert d1 <= d2
    assert d1 <= d3


def test_less_than_or_equal_false():
    d1 = pendulum.datetime(2000, 1, 2)
    d2 = pendulum.datetime(2000, 1, 1)
    d3 = datetime(2000, 1, 1, tzinfo=pendulum.UTC)

    assert not d1 <= d2
    assert not d1 <= d3


def test_less_than_or_equal_with_timezone_true():
    d1 = pendulum.datetime(2000, 1, 1, 8, 59, 59, tz="America/Vancouver")
    d2 = pendulum.datetime(2000, 1, 1, 12, 0, 0, tz="America/Toronto")
    d3 = pytz.timezone("America/Toronto").localize(datetime(2000, 1, 1, 12, 0, 0))

    assert d1 <= d2
    assert d1 <= d3


def test_less_than_or_equal_with_timezone_false():
    d1 = pendulum.datetime(2000, 1, 1, 9, 0, 1, tz="America/Vancouver")
    d2 = pendulum.datetime(2000, 1, 1, 12, 0, 0, tz="America/Toronto")
    d3 = pytz.timezone("America/Toronto").localize(datetime(2000, 1, 1, 12, 0, 0))

    assert not d1 <= d2
    assert not d1 <= d3


def test_is_anniversary():
    with pendulum.travel_to(pendulum.now()):
        d = pendulum.now()
        an_anniversary = d.subtract(years=1)
        assert an_anniversary.is_anniversary()
        not_an_anniversary = d.subtract(days=1)
        assert not not_an_anniversary.is_anniversary()
        also_not_an_anniversary = d.add(days=2)
        assert not also_not_an_anniversary.is_anniversary()

    d1 = pendulum.datetime(1987, 4, 23)
    d2 = pendulum.datetime(2014, 9, 26)
    d3 = pendulum.datetime(2014, 4, 23)
    assert not d2.is_anniversary(d1)
    assert d3.is_anniversary(d1)


def test_is_birthday():  # backward compatibility
    with pendulum.travel_to(pendulum.now()):
        d = pendulum.now()
        an_anniversary = d.subtract(years=1)
        assert an_anniversary.is_birthday()
        not_an_anniversary = d.subtract(days=1)
        assert not not_an_anniversary.is_birthday()
        also_not_an_anniversary = d.add(days=2)
        assert not also_not_an_anniversary.is_birthday()

    d1 = pendulum.datetime(1987, 4, 23)
    d2 = pendulum.datetime(2014, 9, 26)
    d3 = pendulum.datetime(2014, 4, 23)
    assert not d2.is_birthday(d1)
    assert d3.is_birthday(d1)


def test_closest():
    instance = pendulum.datetime(2015, 5, 28, 12, 0, 0)
    dt1 = pendulum.datetime(2015, 5, 28, 11, 0, 0)
    dt2 = pendulum.datetime(2015, 5, 28, 14, 0, 0)
    closest = instance.closest(dt1, dt2)
    assert closest == dt1

    closest = instance.closest(dt2, dt1)
    assert closest == dt1

    dts = [
        pendulum.datetime(2015, 5, 28, 16, 0, 0) + pendulum.duration(hours=x)
        for x in range(4)
    ]
    closest = instance.closest(*dts)
    assert closest == dts[0]

    closest = instance.closest(*(dts[::-1]))
    assert closest == dts[0]


def test_closest_with_datetime():
    instance = pendulum.datetime(2015, 5, 28, 12, 0, 0)
    dt1 = datetime(2015, 5, 28, 11, 0, 0)
    dt2 = datetime(2015, 5, 28, 14, 0, 0)
    closest = instance.closest(dt1, dt2)
    assert_datetime(closest, 2015, 5, 28, 11, 0, 0)

    dts = [
        pendulum.datetime(2015, 5, 28, 16, 0, 0) + pendulum.duration(hours=x)
        for x in range(4)
    ]
    closest = instance.closest(dt1, dt2, *dts)

    assert_datetime(closest, 2015, 5, 28, 11, 0, 0)


def test_closest_with_equals():
    instance = pendulum.datetime(2015, 5, 28, 12, 0, 0)
    dt1 = pendulum.datetime(2015, 5, 28, 12, 0, 0)
    dt2 = pendulum.datetime(2015, 5, 28, 14, 0, 0)
    closest = instance.closest(dt1, dt2)
    assert closest == dt1


def test_farthest():
    instance = pendulum.datetime(2015, 5, 28, 12, 0, 0)
    dt1 = pendulum.datetime(2015, 5, 28, 11, 0, 0)
    dt2 = pendulum.datetime(2015, 5, 28, 14, 0, 0)
    farthest = instance.farthest(dt1, dt2)
    assert farthest == dt2

    farthest = instance.farthest(dt2, dt1)
    assert farthest == dt2

    dts = [
        pendulum.datetime(2015, 5, 28, 16, 0, 0) + pendulum.duration(hours=x)
        for x in range(4)
    ]
    farthest = instance.farthest(*dts)
    assert farthest == dts[-1]

    farthest = instance.farthest(*(dts[::-1]))
    assert farthest == dts[-1]

    f = pendulum.datetime(2010, 1, 1, 0, 0, 0)
    assert f == instance.farthest(f, *(dts))


def test_farthest_with_datetime():
    instance = pendulum.datetime(2015, 5, 28, 12, 0, 0)
    dt1 = datetime(2015, 5, 28, 11, 0, 0, tzinfo=pendulum.UTC)
    dt2 = datetime(2015, 5, 28, 14, 0, 0, tzinfo=pendulum.UTC)
    farthest = instance.farthest(dt1, dt2)
    assert_datetime(farthest, 2015, 5, 28, 14, 0, 0)

    dts = [
        pendulum.datetime(2015, 5, 28, 16, 0, 0) + pendulum.duration(hours=x)
        for x in range(4)
    ]
    farthest = instance.farthest(dt1, dt2, *dts)

    assert_datetime(farthest, 2015, 5, 28, 19, 0, 0)


def test_farthest_with_equals():
    instance = pendulum.datetime(2015, 5, 28, 12, 0, 0)
    dt1 = pendulum.datetime(2015, 5, 28, 12, 0, 0)
    dt2 = pendulum.datetime(2015, 5, 28, 14, 0, 0)
    farthest = instance.farthest(dt1, dt2)
    assert farthest == dt2

    dts = [
        pendulum.datetime(2015, 5, 28, 16, 0, 0) + pendulum.duration(hours=x)
        for x in range(4)
    ]
    farthest = instance.farthest(dt1, dt2, *dts)
    assert farthest == dts[-1]


def test_is_same_day():
    dt1 = pendulum.datetime(2015, 5, 28, 12, 0, 0)
    dt2 = pendulum.datetime(2015, 5, 29, 12, 0, 0)
    dt3 = pendulum.datetime(2015, 5, 28, 12, 0, 0)
    dt4 = datetime(2015, 5, 28, 12, 0, 0, tzinfo=pendulum.UTC)
    dt5 = datetime(2015, 5, 29, 12, 0, 0, tzinfo=pendulum.UTC)

    assert not dt1.is_same_day(dt2)
    assert dt1.is_same_day(dt3)
    assert dt1.is_same_day(dt4)
    assert not dt1.is_same_day(dt5)


def test_comparison_to_unsupported():
    dt1 = pendulum.now()

    assert dt1 != "test"
    assert dt1 not in ["test"]



from __future__ import annotations

from datetime import datetime

import pendulum

from tests.conftest import assert_datetime


def test_fluid_year_setter():
    d = pendulum.now()
    new = d.set(year=1995)
    assert isinstance(new, datetime)
    assert new.year == 1995
    assert d.year != new.year


def test_fluid_month_setter():
    d = pendulum.datetime(2016, 7, 2, 0, 41, 20)
    new = d.set(month=11)
    assert isinstance(new, datetime)
    assert new.month == 11
    assert d.month == 7


def test_fluid_day_setter():
    d = pendulum.datetime(2016, 7, 2, 0, 41, 20)
    new = d.set(day=9)
    assert isinstance(new, datetime)
    assert new.day == 9
    assert d.day == 2


def test_fluid_hour_setter():
    d = pendulum.datetime(2016, 7, 2, 0, 41, 20)
    new = d.set(hour=5)
    assert isinstance(new, datetime)
    assert new.hour == 5
    assert d.hour == 0


def test_fluid_minute_setter():
    d = pendulum.datetime(2016, 7, 2, 0, 41, 20)
    new = d.set(minute=32)
    assert isinstance(new, datetime)
    assert new.minute == 32
    assert d.minute == 41


def test_fluid_second_setter():
    d = pendulum.datetime(2016, 7, 2, 0, 41, 20)
    new = d.set(second=49)
    assert isinstance(new, datetime)
    assert new.second == 49
    assert d.second == 20


def test_fluid_microsecond_setter():
    d = pendulum.datetime(2016, 7, 2, 0, 41, 20, 123456)
    new = d.set(microsecond=987654)
    assert isinstance(new, datetime)
    assert new.microsecond == 987654
    assert d.microsecond == 123456


def test_fluid_setter_keeps_timezone():
    d = pendulum.datetime(2016, 7, 2, 0, 41, 20, 123456, tz="Europe/Paris")
    new = d.set(microsecond=987654)
    assert_datetime(new, 2016, 7, 2, 0, 41, 20, 987654)


def test_fluid_timezone_setter():
    d = pendulum.datetime(2016, 7, 2, 0, 41, 20)
    new = d.set(tz="Europe/Paris")
    assert isinstance(new, datetime)
    assert new.timezone_name == "Europe/Paris"
    assert new.tzinfo.name == "Europe/Paris"


def test_fluid_on():
    d = pendulum.datetime(2016, 7, 2, 0, 41, 20)
    new = d.on(1995, 11, 9)
    assert isinstance(new, datetime)
    assert new.year == 1995
    assert new.month == 11
    assert new.day == 9
    assert d.year == 2016
    assert d.month == 7
    assert d.day == 2


def test_fluid_on_with_transition():
    d = pendulum.datetime(2013, 3, 31, 0, 0, 0, 0, tz="Europe/Paris")
    new = d.on(2013, 4, 1)
    assert isinstance(new, datetime)
    assert new.year == 2013
    assert new.month == 4
    assert new.day == 1
    assert new.offset == 7200
    assert d.year == 2013
    assert d.month == 3
    assert d.day == 31
    assert d.offset == 3600


def test_fluid_at():
    d = pendulum.datetime(2016, 7, 2, 0, 41, 20)
    new = d.at(5, 32, 49, 123456)
    assert isinstance(new, datetime)
    assert new.hour == 5
    assert new.minute == 32
    assert new.second == 49
    assert new.microsecond == 123456
    assert d.hour == 0
    assert d.minute == 41
    assert d.second == 20
    assert d.microsecond == 0


def test_fluid_at_partial():
    d = pendulum.datetime(2016, 7, 2, 0, 41, 20)
    new = d.at(10)

    assert_datetime(new, 2016, 7, 2, 10, 0, 0, 0)

    new = d.at(10, 30)

    assert_datetime(new, 2016, 7, 2, 10, 30, 0, 0)

    new = d.at(10, 30, 45)

    assert_datetime(new, 2016, 7, 2, 10, 30, 45, 0)


def test_fluid_at_with_transition():
    d = pendulum.datetime(2013, 3, 31, 0, 0, 0, 0, tz="Europe/Paris")
    new = d.at(2, 30, 0)
    assert isinstance(new, datetime)
    assert new.hour == 3
    assert new.minute == 30
    assert new.second == 0


def test_replace_tzinfo_dst_off():
    d = pendulum.datetime(2016, 3, 27, 0, 30)  # 30 min before DST turning on
    new = d.replace(tzinfo=pendulum.timezone("Europe/Paris"))

    assert_datetime(new, 2016, 3, 27, 0, 30)
    assert not new.is_dst()
    assert new.offset == 3600
    assert new.timezone_name == "Europe/Paris"


def test_replace_tzinfo_dst_transitioning_on():
    d = pendulum.datetime(2016, 3, 27, 1, 30)  # In middle of turning on
    new = d.replace(tzinfo=pendulum.timezone("Europe/Paris"))

    assert_datetime(new, 2016, 3, 27, 1, 30)
    assert not new.is_dst()
    assert new.offset == 3600
    assert new.timezone_name == "Europe/Paris"


def test_replace_tzinfo_dst_on():
    d = pendulum.datetime(2016, 10, 30, 0, 30)  # 30 min before DST turning off
    new = d.replace(tzinfo=pendulum.timezone("Europe/Paris"))

    assert_datetime(new, 2016, 10, 30, 0, 30)
    assert new.is_dst()
    assert new.offset == 7200
    assert new.timezone_name == "Europe/Paris"


def test_replace_tzinfo_dst_transitioning_off():
    d = pendulum.datetime(2016, 10, 30, 1, 30)  # In the middle of turning off
    new = d.replace(tzinfo=pendulum.timezone("Europe/Paris"))

    assert_datetime(new, 2016, 10, 30, 1, 30)
    assert new.is_dst()
    assert new.offset == 7200
    assert new.timezone_name == "Europe/Paris"



from __future__ import annotations

import pytest

from pendulum import timezone
from pendulum.tz.exceptions import InvalidTimezone
from pendulum.tz.timezone import FixedTimezone
from pendulum.tz.timezone import Timezone


def test_timezone_with_name():
    tz = timezone("Europe/Paris")

    assert isinstance(tz, Timezone)
    assert tz.name == "Europe/Paris"


def test_timezone_with_invalid_name():
    with pytest.raises(InvalidTimezone):
        timezone("Invalid")


def test_timezone_with_offset():
    tz = timezone(-19800)

    assert isinstance(tz, FixedTimezone)
    assert tz.name == "-05:30"



from __future__ import annotations

from typing import TYPE_CHECKING


if TYPE_CHECKING:
    from datetime import datetime


class TimezoneError(ValueError):
    pass


class InvalidTimezone(TimezoneError):
    pass


class NonExistingTime(TimezoneError):
    message = "The datetime {} does not exist."

    def __init__(self, dt: datetime) -> None:
        message = self.message.format(dt)

        super().__init__(message)


class AmbiguousTime(TimezoneError):
    message = "The datetime {} is ambiguous."

    def __init__(self, dt: datetime) -> None:
        message = self.message.format(dt)

        super().__init__(message)



from __future__ import annotations

import pendulum

from pendulum import timezone
from tests.conftest import assert_datetime


def test_create_from_timestamp_returns_pendulum():
    d = pendulum.from_timestamp(pendulum.datetime(1975, 5, 21, 22, 32, 5).timestamp())
    assert_datetime(d, 1975, 5, 21, 22, 32, 5)
    assert d.timezone_name == "UTC"


def test_create_from_timestamp_with_timezone_string():
    d = pendulum.from_timestamp(0, "America/Toronto")
    assert d.timezone_name == "America/Toronto"
    assert_datetime(d, 1969, 12, 31, 19, 0, 0)


def test_create_from_timestamp_with_timezone():
    d = pendulum.from_timestamp(0, timezone("America/Toronto"))
    assert d.timezone_name == "America/Toronto"
    assert_datetime(d, 1969, 12, 31, 19, 0, 0)



from __future__ import annotations

import pendulum


locale = "ru"


def test_diff_for_humans():
    with pendulum.travel_to(pendulum.datetime(2016, 8, 29), freeze=True):
        diff_for_humans()


def diff_for_humans():
    d = pendulum.now().subtract(seconds=1)
    assert d.diff_for_humans(locale=locale) == "1 секунду назад"

    d = pendulum.now().subtract(seconds=2)
    assert d.diff_for_humans(locale=locale) == "2 секунды назад"

    d = pendulum.now().subtract(seconds=5)
    assert d.diff_for_humans(locale=locale) == "5 секунд назад"

    d = pendulum.now().subtract(seconds=21)
    assert d.diff_for_humans(locale=locale) == "21 секунду назад"

    d = pendulum.now().subtract(minutes=1)
    assert d.diff_for_humans(locale=locale) == "1 минуту назад"

    d = pendulum.now().subtract(minutes=2)
    assert d.diff_for_humans(locale=locale) == "2 минуты назад"

    d = pendulum.now().subtract(minutes=5)
    assert d.diff_for_humans(locale=locale) == "5 минут назад"

    d = pendulum.now().subtract(hours=1)
    assert d.diff_for_humans(locale=locale) == "1 час назад"

    d = pendulum.now().subtract(hours=2)
    assert d.diff_for_humans(locale=locale) == "2 часа назад"

    d = pendulum.now().subtract(hours=5)
    assert d.diff_for_humans(locale=locale) == "5 часов назад"

    d = pendulum.now().subtract(days=1)
    assert d.diff_for_humans(locale=locale) == "1 день назад"

    d = pendulum.now().subtract(days=2)
    assert d.diff_for_humans(locale=locale) == "2 дня назад"

    d = pendulum.now().subtract(days=5)
    assert d.diff_for_humans(locale=locale) == "5 дней назад"

    d = pendulum.now().subtract(weeks=1)
    assert d.diff_for_humans(locale=locale) == "1 неделю назад"

    d = pendulum.now().subtract(weeks=2)
    assert d.diff_for_humans(locale=locale) == "2 недели назад"

    d = pendulum.now().subtract(months=1)
    assert d.diff_for_humans(locale=locale) == "1 месяц назад"

    d = pendulum.now().subtract(months=2)
    assert d.diff_for_humans(locale=locale) == "2 месяца назад"

    d = pendulum.now().subtract(months=5)
    assert d.diff_for_humans(locale=locale) == "5 месяцев назад"

    d = pendulum.now().subtract(years=1)
    assert d.diff_for_humans(locale=locale) == "1 год назад"

    d = pendulum.now().subtract(years=2)
    assert d.diff_for_humans(locale=locale) == "2 года назад"

    d = pendulum.now().subtract(years=5)
    assert d.diff_for_humans(locale=locale) == "5 лет назад"

    d = pendulum.now().add(seconds=1)
    assert d.diff_for_humans(locale=locale) == "через 1 секунду"

    d = pendulum.now().add(seconds=1)
    d2 = pendulum.now()
    assert d.diff_for_humans(d2, locale=locale) == "1 секунда после"
    assert d2.diff_for_humans(d, locale=locale) == "1 секунда до"

    assert d.diff_for_humans(d2, True, locale=locale) == "1 секунда"
    assert d2.diff_for_humans(d.add(seconds=1), True, locale=locale) == "2 секунды"



from __future__ import annotations

import pendulum


def test_to_string():
    d = pendulum.Date(2016, 10, 16)
    assert str(d) == "2016-10-16"


def test_to_date_string():
    d = pendulum.Date(1975, 12, 25)
    assert d.to_date_string() == "1975-12-25"


def test_to_formatted_date_string():
    d = pendulum.Date(1975, 12, 25)
    assert d.to_formatted_date_string() == "Dec 25, 1975"


def test_repr():
    d = pendulum.Date(1975, 12, 25)

    assert repr(d) == "Date(1975, 12, 25)"
    assert d.__repr__() == "Date(1975, 12, 25)"


def test_format_with_locale():
    d = pendulum.Date(1975, 12, 25)
    expected = "jeudi 25e jour de décembre 1975"
    assert d.format("dddd Do [jour de] MMMM YYYY", locale="fr") == expected


def test_strftime():
    d = pendulum.Date(1975, 12, 25)
    assert d.strftime("%d") == "25"


def test_for_json():
    d = pendulum.Date(1975, 12, 25)
    assert d.for_json() == "1975-12-25"


def test_format():
    d = pendulum.Date(1975, 12, 25)
    assert f"{d}" == "1975-12-25"
    assert f"{d:YYYY}" == "1975"
    assert f"{d:%Y}" == "1975"



from __future__ import annotations

import os
import sys

import pytest

from pendulum.tz.local_timezone import _get_unix_timezone
from pendulum.tz.local_timezone import _get_windows_timezone


@pytest.mark.skipif(
    sys.platform == "win32", reason="Test only available for UNIX systems"
)
def test_unix_symlink():
    # A ZONE setting in the target path of a symbolic linked localtime,
    # f ex systemd distributions
    local_path = os.path.join(os.path.split(__file__)[0], "..")
    tz = _get_unix_timezone(_root=os.path.join(local_path, "fixtures", "tz", "symlink"))

    assert tz.name == "Europe/Paris"


@pytest.mark.skipif(
    sys.platform == "win32", reason="Test only available for UNIX systems"
)
def test_unix_clock():
    # A ZONE setting in the target path of a symbolic linked localtime,
    # f ex systemd distributions
    local_path = os.path.join(os.path.split(__file__)[0], "..")
    tz = _get_unix_timezone(_root=os.path.join(local_path, "fixtures", "tz", "clock"))

    assert tz.name == "Europe/Zurich"


@pytest.mark.skipif(sys.platform != "win32", reason="Test only available for Windows")
def test_windows_timezone():
    timezone = _get_windows_timezone()

    assert timezone is not None


@pytest.mark.skipif(
    sys.platform == "win32", reason="Test only available for UNIX systems"
)
def test_unix_etc_timezone_dir():
    # Should not fail if `/etc/timezone` is a folder
    local_path = os.path.join(os.path.split(__file__)[0], "..")
    root_path = os.path.join(local_path, "fixtures", "tz", "timezone_dir")
    tz = _get_unix_timezone(_root=root_path)

    assert tz.name == "Europe/Paris"



from __future__ import annotations

import pickle

from datetime import date

import pytest

import pendulum


@pytest.fixture()
def p():
    return pendulum.Date(2016, 8, 27)


@pytest.fixture()
def d():
    return date(2016, 8, 27)


def test_timetuple(p, d):
    assert p.timetuple() == d.timetuple()


def test_ctime(p, d):
    assert p.ctime() == d.ctime()


def test_isoformat(p, d):
    assert p.isoformat() == d.isoformat()


def test_toordinal(p, d):
    assert p.toordinal() == d.toordinal()


def test_weekday(p, d):
    assert p.weekday() == d.weekday()


def test_isoweekday(p, d):
    assert p.isoweekday() == d.isoweekday()


def test_isocalendar(p, d):
    assert p.isocalendar() == d.isocalendar()


def test_fromtimestamp():
    assert pendulum.Date.fromtimestamp(0) == date.fromtimestamp(0)


def test_fromordinal():
    assert pendulum.Date.fromordinal(730120) == date.fromordinal(730120)


def test_hash():
    d1 = pendulum.Date(2016, 8, 27)
    d2 = pendulum.Date(2016, 8, 27)
    d3 = pendulum.Date(2016, 8, 28)

    assert hash(d2) == hash(d1)
    assert hash(d1) != hash(d3)


def test_pickle():
    d1 = pendulum.Date(2016, 8, 27)
    s = pickle.dumps(d1)
    d2 = pickle.loads(s)

    assert isinstance(d2, pendulum.Date)
    assert d2 == d1



from __future__ import annotations

import pendulum


locale = "ja"


def test_diff_for_humans():
    with pendulum.travel_to(pendulum.datetime(2016, 8, 29), freeze=True):
        diff_for_humans()


def diff_for_humans():
    d = pendulum.now().subtract(seconds=1)
    assert d.diff_for_humans(locale=locale) == "数秒 前に"

    d = pendulum.now().subtract(seconds=2)
    assert d.diff_for_humans(locale=locale) == "数秒 前に"

    d = pendulum.now().subtract(seconds=21)
    assert d.diff_for_humans(locale=locale) == "21 秒前"

    d = pendulum.now().subtract(minutes=1)
    assert d.diff_for_humans(locale=locale) == "1 分前"

    d = pendulum.now().subtract(minutes=2)
    assert d.diff_for_humans(locale=locale) == "2 分前"

    d = pendulum.now().subtract(hours=1)
    assert d.diff_for_humans(locale=locale) == "1 時間前"

    d = pendulum.now().subtract(hours=2)
    assert d.diff_for_humans(locale=locale) == "2 時間前"

    d = pendulum.now().subtract(days=1)
    assert d.diff_for_humans(locale=locale) == "1 日前"

    d = pendulum.now().subtract(days=2)
    assert d.diff_for_humans(locale=locale) == "2 日前"

    d = pendulum.now().subtract(weeks=1)
    assert d.diff_for_humans(locale=locale) == "1 週間前"

    d = pendulum.now().subtract(weeks=2)
    assert d.diff_for_humans(locale=locale) == "2 週間前"

    d = pendulum.now().subtract(months=1)
    assert d.diff_for_humans(locale=locale) == "1 か月前"

    d = pendulum.now().subtract(months=2)
    assert d.diff_for_humans(locale=locale) == "2 か月前"

    d = pendulum.now().subtract(years=1)
    assert d.diff_for_humans(locale=locale) == "1 年前"

    d = pendulum.now().subtract(years=2)
    assert d.diff_for_humans(locale=locale) == "2 年前"

    d = pendulum.now().add(seconds=1)
    assert d.diff_for_humans(locale=locale) == "今から 数秒"

    d = pendulum.now().add(seconds=1)
    d2 = pendulum.now()
    assert d.diff_for_humans(d2, locale=locale) == "数秒 後"
    assert d2.diff_for_humans(d, locale=locale) == "数秒 前"

    assert d.diff_for_humans(d2, True, locale=locale) == "数秒"
    assert d2.diff_for_humans(d.add(seconds=1), True, locale=locale) == "数秒"



from __future__ import annotations

from typing import TYPE_CHECKING
from typing import cast

from pendulum.datetime import DateTime
from pendulum.utils._compat import PYPY


if TYPE_CHECKING:
    from types import TracebackType

    from typing_extensions import Self


class BaseTraveller:
    def __init__(self, datetime_class: type[DateTime] = DateTime) -> None:
        self._datetime_class: type[DateTime] = datetime_class

    def freeze(self) -> Self:
        raise self._not_implemented()

    def travel_back(self) -> Self:
        raise self._not_implemented()

    def travel(
        self,
        years: int = 0,
        months: int = 0,
        weeks: int = 0,
        days: int = 0,
        hours: int = 0,
        minutes: int = 0,
        seconds: int = 0,
        microseconds: int = 0,
    ) -> Self:
        raise self._not_implemented()

    def travel_to(self, dt: DateTime, *, freeze: bool = False) -> Self:
        raise self._not_implemented()

    def __enter__(self) -> Self:
        return self

    def __exit__(
        self,
        exc_type: type[BaseException] | None,
        exc_val: BaseException | None,
        exc_tb: TracebackType,
    ) -> None: ...

    def _not_implemented(self) -> NotImplementedError:
        return NotImplementedError()


if not PYPY:
    try:
        import time_machine
    except ImportError:
        time_machine = None  # type: ignore[assignment]

    if time_machine is not None:

        class Traveller(BaseTraveller):
            def __init__(self, datetime_class: type[DateTime] = DateTime) -> None:
                super().__init__(datetime_class)

                self._started: bool = False
                self._traveller: time_machine.travel | None = None
                self._coordinates: time_machine.Coordinates | None = None

            def freeze(self) -> Self:
                if self._started:
                    cast("time_machine.Coordinates", self._coordinates).move_to(
                        self._datetime_class.now(), tick=False
                    )
                else:
                    self._start(freeze=True)

                return self

            def travel_back(self) -> Self:
                if not self._started:
                    return self

                cast("time_machine.travel", self._traveller).stop()
                self._coordinates = None
                self._traveller = None
                self._started = False

                return self

            def travel(
                self,
                years: int = 0,
                months: int = 0,
                weeks: int = 0,
                days: int = 0,
                hours: int = 0,
                minutes: int = 0,
                seconds: int = 0,
                microseconds: int = 0,
                *,
                freeze: bool = False,
            ) -> Self:
                self._start(freeze=freeze)

                cast("time_machine.Coordinates", self._coordinates).move_to(
                    self._datetime_class.now().add(
                        years=years,
                        months=months,
                        weeks=weeks,
                        days=days,
                        hours=hours,
                        minutes=minutes,
                        seconds=seconds,
                        microseconds=microseconds,
                    )
                )

                return self

            def travel_to(self, dt: DateTime, *, freeze: bool = False) -> Self:
                self._start(freeze=freeze)

                cast("time_machine.Coordinates", self._coordinates).move_to(dt)

                return self

            def _start(self, freeze: bool = False) -> None:
                if self._started:
                    return

                if not self._traveller:
                    self._traveller = time_machine.travel(
                        self._datetime_class.now(), tick=not freeze
                    )

                self._coordinates = self._traveller.start()

                self._started = True

            def __enter__(self) -> Self:
                self._start()

                return self

            def __exit__(
                self,
                exc_type: type[BaseException] | None,
                exc_val: BaseException | None,
                exc_tb: TracebackType,
            ) -> None:
                self.travel_back()

    else:

        class Traveller(BaseTraveller):  # type: ignore[no-redef]
            def _not_implemented(self) -> NotImplementedError:
                return NotImplementedError(
                    "Time travelling is an optional feature. "
                    'You can add it by installing Pendulum with the "test" extra.'
                )

else:

    class Traveller(BaseTraveller):  # type: ignore[no-redef]
        def _not_implemented(self) -> NotImplementedError:
            return NotImplementedError(
                "Time travelling is not supported on the PyPy Python implementation."
            )



from .custom import translations as custom_translations


"""
bg locale file.

It has been generated automatically and must not be modified directly.
"""


locale = {
    'plural': lambda n: 'one' if (n == n and ((n == 1))) else 'other',
    'ordinal': lambda n: 'other',
    'translations': {
        'days': {
            'abbreviated': {
                0: 'пн',
                1: 'вт',
                2: 'ср',
                3: 'чт',
                4: 'пт',
                5: 'сб',
                6: 'нд',
            },
            'narrow': {
                0: 'п',
                1: 'в',
                2: 'с',
                3: 'ч',
                4: 'п',
                5: 'с',
                6: 'н',
            },
            'short': {
                0: 'пн',
                1: 'вт',
                2: 'ср',
                3: 'чт',
                4: 'пт',
                5: 'сб',
                6: 'нд',
            },
            'wide': {
                0: 'понеделник',
                1: 'вторник',
                2: 'сряда',
                3: 'четвъртък',
                4: 'петък',
                5: 'събота',
                6: 'неделя',
            },
        },
        'months': {
            'abbreviated': {
                1: 'яну',
                2: 'фев',
                3: 'март',
                4: 'апр',
                5: 'май',
                6: 'юни',
                7: 'юли',
                8: 'авг',
                9: 'сеп',
                10: 'окт',
                11: 'ное',
                12: 'дек',
            },
            'narrow': {
                1: 'я',
                2: 'ф',
                3: 'м',
                4: 'а',
                5: 'м',
                6: 'ю',
                7: 'ю',
                8: 'а',
                9: 'с',
                10: 'о',
                11: 'н',
                12: 'д',
            },
            'wide': {
                1: 'януари',
                2: 'февруари',
                3: 'март',
                4: 'април',
                5: 'май',
                6: 'юни',
                7: 'юли',
                8: 'август',
                9: 'септември',
                10: 'октомври',
                11: 'ноември',
                12: 'декември',
            },
        },
        'units': {
            'year': {
                'one': '{0} година',
                'other': '{0} години',
            },
            'month': {
                'one': '{0} месец',
                'other': '{0} месеца',
            },
            'week': {
                'one': '{0} седмица',
                'other': '{0} седмици',
            },
            'day': {
                'one': '{0} ден',
                'other': '{0} дни',
            },
            'hour': {
                'one': '{0} час',
                'other': '{0} часа',
            },
            'minute': {
                'one': '{0} минута',
                'other': '{0} минути',
            },
            'second': {
                'one': '{0} секунда',
                'other': '{0} секунди',
            },
            'microsecond': {
                'one': '{0} микросекунда',
                'other': '{0} микросекунди',
            },
        },
        'relative': {
            'year': {
                'future': {
                    'other': 'след {0} години',
                    'one': 'след {0} година',
                },
                'past': {
                    'other': 'преди {0} години',
                    'one': 'преди {0} година',
                },
            },
            'month': {
                'future': {
                    'other': 'след {0} месеца',
                    'one': 'след {0} месец',
                },
                'past': {
                    'other': 'преди {0} месеца',
                    'one': 'преди {0} месец',
                },
            },
            'week': {
                'future': {
                    'other': 'след {0} седмици',
                    'one': 'след {0} седмица',
                },
                'past': {
                    'other': 'преди {0} седмици',
                    'one': 'преди {0} седмица',
                },
            },
            'day': {
                'future': {
                    'other': 'след {0} дни',
                    'one': 'след {0} ден',
                },
                'past': {
                    'other': 'преди {0} дни',
                    'one': 'преди {0} ден',
                },
            },
            'hour': {
                'future': {
                    'other': 'след {0} часа',
                    'one': 'след {0} час',
                },
                'past': {
                    'other': 'преди {0} часа',
                    'one': 'преди {0} час',
                },
            },
            'minute': {
                'future': {
                    'other': 'след {0} минути',
                    'one': 'след {0} минута',
                },
                'past': {
                    'other': 'преди {0} минути',
                    'one': 'преди {0} минута',
                },
            },
            'second': {
                'future': {
                    'other': 'след {0} секунди',
                    'one': 'след {0} секунда',
                },
                'past': {
                    'other': 'преди {0} секунди',
                    'one': 'преди {0} секунда',
                },
            },
        },
        'day_periods': {
            'midnight': 'полунощ',
            'am': 'пр.об.',
            'pm': 'сл.об.',
            'morning1': 'сутринта',
            'morning2': 'на обяд',
            'afternoon1': 'следобед',
            'evening1': 'вечерта',
            'night1': 'през нощта',
        },
        'week_data': {
            'min_days': 1,
            'first_day': 0,
            'weekend_start': 5,
            'weekend_end': 6,
        },
    },
    'custom': custom_translations
}



from __future__ import annotations

import pendulum

from tests.conftest import assert_date
from tests.conftest import assert_datetime
from tests.conftest import assert_duration
from tests.conftest import assert_time


def test_parse() -> None:
    text = "2016-10-16T12:34:56.123456+01:30"

    dt = pendulum.parse(text)

    assert isinstance(dt, pendulum.DateTime)
    assert_datetime(dt, 2016, 10, 16, 12, 34, 56, 123456)
    assert dt.tz is not None
    assert dt.tz.name == "+01:30"
    assert dt.offset == 5400

    text = "2016-10-16"

    dt = pendulum.parse(text)

    assert isinstance(dt, pendulum.DateTime)
    assert_datetime(dt, 2016, 10, 16, 0, 0, 0, 0)
    assert dt.offset == 0

    with pendulum.travel_to(pendulum.datetime(2015, 11, 12), freeze=True):
        text = "12:34:56.123456"

        dt = pendulum.parse(text)

    assert isinstance(dt, pendulum.DateTime)
    assert_datetime(dt, 2015, 11, 12, 12, 34, 56, 123456)
    assert dt.offset == 0


def test_parse_with_timezone() -> None:
    text = "2016-10-16T12:34:56.123456"

    dt = pendulum.parse(text, tz="Europe/Paris")
    assert isinstance(dt, pendulum.DateTime)
    assert_datetime(dt, 2016, 10, 16, 12, 34, 56, 123456)
    assert dt.tz is not None
    assert dt.tz.name == "Europe/Paris"
    assert dt.offset == 7200


def test_parse_exact() -> None:
    text = "2016-10-16T12:34:56.123456+01:30"

    dt = pendulum.parse(text, exact=True)

    assert isinstance(dt, pendulum.DateTime)
    assert_datetime(dt, 2016, 10, 16, 12, 34, 56, 123456)
    assert dt.offset == 5400

    text = "2016-10-16"

    dt = pendulum.parse(text, exact=True)

    assert isinstance(dt, pendulum.Date)
    assert_date(dt, 2016, 10, 16)

    text = "12:34:56.123456"

    dt = pendulum.parse(text, exact=True)

    assert isinstance(dt, pendulum.Time)
    assert_time(dt, 12, 34, 56, 123456)

    text = "13:00"

    dt = pendulum.parse(text, exact=True)

    assert isinstance(dt, pendulum.Time)
    assert_time(dt, 13, 0, 0)


def test_parse_duration() -> None:
    text = "P2Y3M4DT5H6M7S"

    duration = pendulum.parse(text)

    assert isinstance(duration, pendulum.Duration)
    assert_duration(duration, 2, 3, 0, 4, 5, 6, 7)

    text = "P2W"

    duration = pendulum.parse(text)

    assert isinstance(duration, pendulum.Duration)
    assert_duration(duration, 0, 0, 2, 0, 0, 0, 0)


def test_parse_interval() -> None:
    text = "2008-05-11T15:30:00Z/P1Y2M10DT2H30M"

    interval = pendulum.parse(text)

    assert isinstance(interval, pendulum.Interval)
    assert_datetime(interval.start, 2008, 5, 11, 15, 30, 0, 0)
    assert interval.start.offset == 0
    assert_datetime(interval.end, 2009, 7, 21, 18, 0, 0, 0)
    assert interval.end.offset == 0

    text = "P1Y2M10DT2H30M/2008-05-11T15:30:00Z"

    interval = pendulum.parse(text)

    assert isinstance(interval, pendulum.Interval)
    assert_datetime(interval.start, 2007, 3, 1, 13, 0, 0, 0)
    assert interval.start.offset == 0
    assert_datetime(interval.end, 2008, 5, 11, 15, 30, 0, 0)
    assert interval.end.offset == 0

    text = "2007-03-01T13:00:00Z/2008-05-11T15:30:00Z"

    interval = pendulum.parse(text)

    assert isinstance(interval, pendulum.Interval)
    assert_datetime(interval.start, 2007, 3, 1, 13, 0, 0, 0)
    assert interval.start.offset == 0
    assert_datetime(interval.end, 2008, 5, 11, 15, 30, 0, 0)
    assert interval.end.offset == 0


def test_parse_now() -> None:
    dt = pendulum.parse("now")

    assert dt.timezone_name == "America/Toronto"

    mock_now = pendulum.yesterday()

    with pendulum.travel_to(mock_now, freeze=True):
        assert pendulum.parse("now") == mock_now


def test_parse_with_utc_timezone() -> None:
    dt = pendulum.parse("2020-02-05T20:05:37.364951Z")

    assert dt.to_iso8601_string() == "2020-02-05T20:05:37.364951Z"



from __future__ import annotations

import pendulum


locale = "he"


def test_diff_for_humans():
    with pendulum.travel_to(pendulum.datetime(2016, 8, 29), freeze=True):
        diff_for_humans()


def diff_for_humans():
    d = pendulum.now().subtract(seconds=1)
    assert d.diff_for_humans(locale=locale) == "לפני כמה שניות"

    d = pendulum.now().subtract(seconds=2)
    assert d.diff_for_humans(locale=locale) == "לפני כמה שניות"

    d = pendulum.now().subtract(minutes=1)
    assert d.diff_for_humans(locale=locale) == "לפני דקה"

    d = pendulum.now().subtract(minutes=2)
    assert d.diff_for_humans(locale=locale) == "לפני שתי דקות"

    d = pendulum.now().subtract(hours=1)
    assert d.diff_for_humans(locale=locale) == "לפני שעה"

    d = pendulum.now().subtract(hours=2)
    assert d.diff_for_humans(locale=locale) == "לפני שעתיים"

    d = pendulum.now().subtract(days=1)
    assert d.diff_for_humans(locale=locale) == "לפני יום 1"

    d = pendulum.now().subtract(days=2)
    assert d.diff_for_humans(locale=locale) == "לפני יומיים"

    d = pendulum.now().subtract(weeks=1)
    assert d.diff_for_humans(locale=locale) == "לפני שבוע"

    d = pendulum.now().subtract(weeks=2)
    assert d.diff_for_humans(locale=locale) == "לפני שבועיים"

    d = pendulum.now().subtract(months=1)
    assert d.diff_for_humans(locale=locale) == "לפני חודש"

    d = pendulum.now().subtract(months=2)
    assert d.diff_for_humans(locale=locale) == "לפני חודשיים"

    d = pendulum.now().subtract(years=1)
    assert d.diff_for_humans(locale=locale) == "לפני שנה"

    d = pendulum.now().subtract(years=2)
    assert d.diff_for_humans(locale=locale) == "לפני שנתיים"

    d = pendulum.now().add(seconds=1)
    assert d.diff_for_humans(locale=locale) == "תוך כמה שניות"

    d = pendulum.now().add(seconds=1)
    d2 = pendulum.now()
    assert d.diff_for_humans(d2, locale=locale) == "בעוד כמה שניות"
    assert d2.diff_for_humans(d, locale=locale) == "כמה שניות קודם"

    assert d.diff_for_humans(d2, True, locale=locale) == "כמה שניות"
    assert d2.diff_for_humans(d.add(seconds=1), True, locale=locale) == "כמה שניות"



from __future__ import annotations

from pendulum.locales.cs.custom import translations as custom_translations


"""
cs locale file.

It has been generated automatically and must not be modified directly.
"""


locale = {
    "plural": lambda n: "few"
    if ((n == n and (n >= 2 and n <= 4)) and (0 == 0 and (0 == 0)))
    else "many"
    if (not (0 == 0 and (0 == 0)))
    else "one"
    if ((n == n and (n == 1)) and (0 == 0 and (0 == 0)))
    else "other",
    "ordinal": lambda n: "other",
    "translations": {
        "days": {
            "abbreviated": {
                0: "po",
                1: "út",
                2: "st",
                3: "čt",
                4: "pá",
                5: "so",
                6: "ne",
            },
            "narrow": {
                0: "P",
                1: "Ú",
                2: "S",
                3: "Č",
                4: "P",
                5: "S",
                6: "N",
            },
            "short": {
                0: "po",
                1: "út",
                2: "st",
                3: "čt",
                4: "pá",
                5: "so",
                6: "ne",
            },
            "wide": {
                0: "pondělí",
                1: "úterý",
                2: "středa",
                3: "čtvrtek",
                4: "pátek",
                5: "sobota",
                6: "neděle",
            },
        },
        "months": {
            "abbreviated": {
                1: "led",
                2: "úno",
                3: "bře",
                4: "dub",
                5: "kvě",
                6: "čvn",
                7: "čvc",
                8: "srp",
                9: "zář",
                10: "říj",
                11: "lis",
                12: "pro",
            },
            "narrow": {
                1: "1",
                2: "2",
                3: "3",
                4: "4",
                5: "5",
                6: "6",
                7: "7",
                8: "8",
                9: "9",
                10: "10",
                11: "11",
                12: "12",
            },
            "wide": {
                1: "ledna",
                2: "února",
                3: "března",
                4: "dubna",
                5: "května",
                6: "června",
                7: "července",
                8: "srpna",
                9: "září",
                10: "října",
                11: "listopadu",
                12: "prosince",
            },
        },
        "units": {
            "year": {
                "one": "{0} rok",
                "few": "{0} roky",
                "many": "{0} roku",
                "other": "{0} let",
            },
            "month": {
                "one": "{0} měsíc",
                "few": "{0} měsíce",
                "many": "{0} měsíce",
                "other": "{0} měsíců",
            },
            "week": {
                "one": "{0} týden",
                "few": "{0} týdny",
                "many": "{0} týdne",
                "other": "{0} týdnů",
            },
            "day": {
                "one": "{0} den",
                "few": "{0} dny",
                "many": "{0} dne",
                "other": "{0} dní",
            },
            "hour": {
                "one": "{0} hodina",
                "few": "{0} hodiny",
                "many": "{0} hodiny",
                "other": "{0} hodin",
            },
            "minute": {
                "one": "{0} minuta",
                "few": "{0} minuty",
                "many": "{0} minuty",
                "other": "{0} minut",
            },
            "second": {
                "one": "{0} sekunda",
                "few": "{0} sekundy",
                "many": "{0} sekundy",
                "other": "{0} sekund",
            },
            "microsecond": {
                "one": "{0} mikrosekunda",
                "few": "{0} mikrosekundy",
                "many": "{0} mikrosekundy",
                "other": "{0} mikrosekund",
            },
        },
        "relative": {
            "year": {
                "future": {
                    "other": "za {0} let",
                    "one": "za {0} rok",
                    "few": "za {0} roky",
                    "many": "za {0} roku",
                },
                "past": {
                    "other": "před {0} lety",
                    "one": "před {0} rokem",
                    "few": "před {0} lety",
                    "many": "před {0} roku",
                },
            },
            "month": {
                "future": {
                    "other": "za {0} měsíců",
                    "one": "za {0} měsíc",
                    "few": "za {0} měsíce",
                    "many": "za {0} měsíce",
                },
                "past": {
                    "other": "před {0} měsíci",
                    "one": "před {0} měsícem",
                    "few": "před {0} měsíci",
                    "many": "před {0} měsíce",
                },
            },
            "week": {
                "future": {
                    "other": "za {0} týdnů",
                    "one": "za {0} týden",
                    "few": "za {0} týdny",
                    "many": "za {0} týdne",
                },
                "past": {
                    "other": "před {0} týdny",
                    "one": "před {0} týdnem",
                    "few": "před {0} týdny",
                    "many": "před {0} týdne",
                },
            },
            "day": {
                "future": {
                    "other": "za {0} dní",
                    "one": "za {0} den",
                    "few": "za {0} dny",
                    "many": "za {0} dne",
                },
                "past": {
                    "other": "před {0} dny",
                    "one": "před {0} dnem",
                    "few": "před {0} dny",
                    "many": "před {0} dne",
                },
            },
            "hour": {
                "future": {
                    "other": "za {0} hodin",
                    "one": "za {0} hodinu",
                    "few": "za {0} hodiny",
                    "many": "za {0} hodiny",
                },
                "past": {
                    "other": "před {0} hodinami",
                    "one": "před {0} hodinou",
                    "few": "před {0} hodinami",
                    "many": "před {0} hodiny",
                },
            },
            "minute": {
                "future": {
                    "other": "za {0} minut",
                    "one": "za {0} minutu",
                    "few": "za {0} minuty",
                    "many": "za {0} minuty",
                },
                "past": {
                    "other": "před {0} minutami",
                    "one": "před {0} minutou",
                    "few": "před {0} minutami",
                    "many": "před {0} minuty",
                },
            },
            "second": {
                "future": {
                    "other": "za {0} sekund",
                    "one": "za {0} sekundu",
                    "few": "za {0} sekundy",
                    "many": "za {0} sekundy",
                },
                "past": {
                    "other": "před {0} sekundami",
                    "one": "před {0} sekundou",
                    "few": "před {0} sekundami",
                    "many": "před {0} sekundy",
                },
            },
        },
        "day_periods": {
            "midnight": "půlnoc",
            "am": "dop.",
            "noon": "poledne",
            "pm": "odp.",
            "morning1": "ráno",
            "morning2": "dopoledne",
            "afternoon1": "odpoledne",
            "evening1": "večer",
            "night1": "v noci",
        },
        "week_data": {
            "min_days": 1,
            "first_day": 0,
            "weekend_start": 5,
            "weekend_end": 6,
        },
    },
    "custom": custom_translations,
}



from __future__ import annotations

import contextlib
import copy
import os
import re

from datetime import date
from datetime import datetime
from datetime import time
from typing import Any
from typing import Optional
from typing import cast

from dateutil import parser

from pendulum.parsing.exceptions import ParserError


with_extensions = os.getenv("PENDULUM_EXTENSIONS", "1") == "1"

try:
    if not with_extensions:
        raise ImportError()

    from pendulum._pendulum import Duration
    from pendulum._pendulum import parse_iso8601
except ImportError:
    from pendulum.duration import Duration  # type: ignore[assignment]  # noqa: TC001
    from pendulum.parsing.iso8601 import parse_iso8601  # type: ignore[assignment]


COMMON = re.compile(
    # Date (optional)  # noqa: ERA001
    "^"
    "(?P<date>"
    "    (?P<classic>"  # Classic date (YYYY-MM-DD)
    r"        (?P<year>\d{4})"  # Year
    "        (?P<monthday>"
    r"            (?P<monthsep>[/:])?(?P<month>\d{2})"  # Month (optional)
    r"            ((?P<daysep>[/:])?(?P<day>\d{2}))"  # Day (optional)
    "        )?"
    "    )"
    ")?"
    # Time (optional)  # noqa: ERA001
    "(?P<time>"
    r"    (?P<timesep>\ )?"  # Separator (space)
    # HH:mm:ss (optional mm and ss)
    r"    (?P<hour>\d{1,2}):(?P<minute>\d{1,2})?(?::(?P<second>\d{1,2}))?"
    # Subsecond part (optional)
    "    (?P<subsecondsection>"
    "        (?:[.|,])"  # Subsecond separator (optional)
    r"        (?P<subsecond>\d{1,9})"  # Subsecond
    "    )?"
    ")?"
    "$",
    re.VERBOSE,
)

DEFAULT_OPTIONS = {
    "day_first": False,
    "year_first": True,
    "strict": True,
    "exact": False,
    "now": None,
}


def parse(text: str, **options: Any) -> datetime | date | time | _Interval | Duration:
    """
    Parses a string with the given options.

    :param text: The string to parse.
    """
    _options: dict[str, Any] = copy.copy(DEFAULT_OPTIONS)
    _options.update(options)

    return _normalize(_parse(text, **_options), **_options)


def _normalize(
    parsed: datetime | date | time | _Interval | Duration, **options: Any
) -> datetime | date | time | _Interval | Duration:
    """
    Normalizes the parsed element.

    :param parsed: The parsed elements.
    """
    if options.get("exact"):
        return parsed

    if isinstance(parsed, time):
        now = cast("Optional[datetime]", options["now"]) or datetime.now()

        return datetime(
            now.year,
            now.month,
            now.day,
            parsed.hour,
            parsed.minute,
            parsed.second,
            parsed.microsecond,
        )
    elif isinstance(parsed, date) and not isinstance(parsed, datetime):
        return datetime(parsed.year, parsed.month, parsed.day)

    return parsed


def _parse(text: str, **options: Any) -> datetime | date | time | _Interval | Duration:
    # Trying to parse ISO8601
    with contextlib.suppress(ValueError):
        return parse_iso8601(text)

    with contextlib.suppress(ValueError):
        return _parse_iso8601_interval(text)

    with contextlib.suppress(ParserError):
        return _parse_common(text, **options)

    # We couldn't parse the string
    # so we fallback on the dateutil parser
    # If not strict
    if options.get("strict", True):
        raise ParserError(f"Unable to parse string [{text}]")

    try:
        dt = parser.parse(
            text, dayfirst=options["day_first"], yearfirst=options["year_first"]
        )
    except ValueError:
        raise ParserError(f"Invalid date string: {text}")

    return dt


def _parse_common(text: str, **options: Any) -> datetime | date | time:
    """
    Tries to parse the string as a common datetime format.

    :param text: The string to parse.
    """
    m = COMMON.match(text)
    has_date = False
    year = 0
    month = 1
    day = 1

    if not m:
        raise ParserError("Invalid datetime string")

    if m.group("date"):
        # A date has been specified
        has_date = True

        year = int(m.group("year"))

        if not m.group("monthday"):
            # No month and day
            month = 1
            day = 1
        else:
            if options["day_first"]:
                month = int(m.group("day"))
                day = int(m.group("month"))
            else:
                month = int(m.group("month"))
                day = int(m.group("day"))

    if not m.group("time"):
        return date(year, month, day)

    # Grabbing hh:mm:ss
    hour = int(m.group("hour"))

    minute = int(m.group("minute"))

    second = int(m.group("second")) if m.group("second") else 0

    # Grabbing subseconds, if any
    microsecond = 0
    if m.group("subsecondsection"):
        # Limiting to 6 chars
        subsecond = m.group("subsecond")[:6]

        microsecond = int(f"{subsecond:0<6}")

    if has_date:
        return datetime(year, month, day, hour, minute, second, microsecond)

    return time(hour, minute, second, microsecond)


class _Interval:
    """
    Special class to handle ISO 8601 intervals
    """

    def __init__(
        self,
        start: datetime | None = None,
        end: datetime | None = None,
        duration: Duration | None = None,
    ) -> None:
        self.start = start
        self.end = end
        self.duration = duration


def _parse_iso8601_interval(text: str) -> _Interval:
    if "/" not in text:
        raise ParserError("Invalid interval")

    first, last = text.split("/")
    start = end = duration = None

    if first[:1] == "P":
        # duration/end
        duration = parse_iso8601(first)
        end = parse_iso8601(last)
    elif last[:1] == "P":
        # start/duration
        start = parse_iso8601(first)
        duration = parse_iso8601(last)
    else:
        # start/end
        start = parse_iso8601(first)
        end = parse_iso8601(last)

    return _Interval(
        cast("datetime", start), cast("datetime", end), cast("Duration", duration)
    )


__all__ = ["parse", "parse_iso8601"]



from __future__ import annotations

import pytest

from pendulum.parsing import ParserError
from pendulum.parsing import parse


def test_parse_duration():
    text = "P2Y3M4DT5H6M7S"
    parsed = parse(text)

    assert parsed.years == 2
    assert parsed.months == 3
    assert parsed.weeks == 0
    assert parsed.remaining_days == 4
    assert parsed.hours == 5
    assert parsed.minutes == 6
    assert parsed.remaining_seconds == 7
    assert parsed.microseconds == 0

    text = "P1Y2M3DT4H5M6.5S"
    parsed = parse(text)

    assert parsed.years == 1
    assert parsed.months == 2
    assert parsed.weeks == 0
    assert parsed.remaining_days == 3
    assert parsed.hours == 4
    assert parsed.minutes == 5
    assert parsed.remaining_seconds == 6
    assert parsed.microseconds == 500000

    text = "P1Y2M3DT4H5M6,5S"
    parsed = parse(text)

    assert parsed.years == 1
    assert parsed.months == 2
    assert parsed.weeks == 0
    assert parsed.remaining_days == 3
    assert parsed.hours == 4
    assert parsed.minutes == 5
    assert parsed.remaining_seconds == 6
    assert parsed.microseconds == 500000

    text = "P1Y2M3D"
    parsed = parse(text)

    assert parsed.years == 1
    assert parsed.months == 2
    assert parsed.weeks == 0
    assert parsed.remaining_days == 3
    assert parsed.hours == 0
    assert parsed.minutes == 0
    assert parsed.remaining_seconds == 0
    assert parsed.microseconds == 0

    text = "P1Y2M3.5D"
    parsed = parse(text)

    assert parsed.years == 1
    assert parsed.months == 2
    assert parsed.weeks == 0
    assert parsed.remaining_days == 3
    assert parsed.hours == 12
    assert parsed.minutes == 0
    assert parsed.remaining_seconds == 0
    assert parsed.microseconds == 0

    text = "P1Y2M3,5D"
    parsed = parse(text)

    assert parsed.years == 1
    assert parsed.months == 2
    assert parsed.weeks == 0
    assert parsed.remaining_days == 3
    assert parsed.hours == 12
    assert parsed.minutes == 0
    assert parsed.remaining_seconds == 0
    assert parsed.microseconds == 0

    text = "PT4H54M6.5S"
    parsed = parse(text)

    assert parsed.years == 0
    assert parsed.months == 0
    assert parsed.weeks == 0
    assert parsed.remaining_days == 0
    assert parsed.hours == 4
    assert parsed.minutes == 54
    assert parsed.remaining_seconds == 6
    assert parsed.microseconds == 500000

    text = "PT4H54M6,5S"
    parsed = parse(text)

    assert parsed.years == 0
    assert parsed.months == 0
    assert parsed.weeks == 0
    assert parsed.remaining_days == 0
    assert parsed.hours == 4
    assert parsed.minutes == 54
    assert parsed.remaining_seconds == 6
    assert parsed.microseconds == 500000

    text = "P1Y"
    parsed = parse(text)

    assert parsed.years == 1
    assert parsed.months == 0
    assert parsed.weeks == 0
    assert parsed.remaining_days == 0
    assert parsed.hours == 0
    assert parsed.minutes == 0
    assert parsed.remaining_seconds == 0
    assert parsed.microseconds == 0

    text = "P1.5Y"
    with pytest.raises(ParserError):
        parse(text)

    text = "P1,5Y"
    with pytest.raises(ParserError):
        parse(text)

    text = "P1M"
    parsed = parse(text)

    assert parsed.years == 0
    assert parsed.months == 1
    assert parsed.weeks == 0
    assert parsed.remaining_days == 0
    assert parsed.hours == 0
    assert parsed.minutes == 0
    assert parsed.remaining_seconds == 0
    assert parsed.microseconds == 0

    text = "P1.5M"
    with pytest.raises(ParserError):
        parse(text)

    text = "P1,5M"
    with pytest.raises(ParserError):
        parse(text)

    text = "P1W"
    parsed = parse(text)

    assert parsed.years == 0
    assert parsed.months == 0
    assert parsed.weeks == 1
    assert parsed.remaining_days == 0
    assert parsed.hours == 0
    assert parsed.minutes == 0
    assert parsed.remaining_seconds == 0
    assert parsed.microseconds == 0

    text = "P1.5W"
    parsed = parse(text)

    assert parsed.years == 0
    assert parsed.months == 0
    assert parsed.weeks == 1
    assert parsed.remaining_days == 3
    assert parsed.hours == 12
    assert parsed.minutes == 0
    assert parsed.remaining_seconds == 0
    assert parsed.microseconds == 0

    text = "P1,5W"
    parsed = parse(text)

    assert parsed.years == 0
    assert parsed.months == 0
    assert parsed.weeks == 1
    assert parsed.remaining_days == 3
    assert parsed.hours == 12
    assert parsed.minutes == 0
    assert parsed.remaining_seconds == 0
    assert parsed.microseconds == 0

    text = "P1D"
    parsed = parse(text)

    assert parsed.years == 0
    assert parsed.months == 0
    assert parsed.weeks == 0
    assert parsed.remaining_days == 1
    assert parsed.hours == 0
    assert parsed.minutes == 0
    assert parsed.remaining_seconds == 0
    assert parsed.microseconds == 0

    text = "P1.5D"
    parsed = parse(text)

    assert parsed.years == 0
    assert parsed.months == 0
    assert parsed.weeks == 0
    assert parsed.remaining_days == 1
    assert parsed.hours == 12
    assert parsed.minutes == 0
    assert parsed.remaining_seconds == 0
    assert parsed.microseconds == 0

    text = "P1,5D"
    parsed = parse(text)

    assert parsed.years == 0
    assert parsed.months == 0
    assert parsed.weeks == 0
    assert parsed.remaining_days == 1
    assert parsed.hours == 12
    assert parsed.minutes == 0
    assert parsed.remaining_seconds == 0
    assert parsed.microseconds == 0

    text = "PT1H"
    parsed = parse(text)

    assert parsed.years == 0
    assert parsed.months == 0
    assert parsed.weeks == 0
    assert parsed.remaining_days == 0
    assert parsed.hours == 1
    assert parsed.minutes == 0
    assert parsed.remaining_seconds == 0
    assert parsed.microseconds == 0

    text = "PT1.5H"
    parsed = parse(text)

    assert parsed.years == 0
    assert parsed.months == 0
    assert parsed.weeks == 0
    assert parsed.remaining_days == 0
    assert parsed.hours == 1
    assert parsed.minutes == 30
    assert parsed.remaining_seconds == 0
    assert parsed.microseconds == 0

    text = "PT1,5H"
    parsed = parse(text)

    assert parsed.years == 0
    assert parsed.months == 0
    assert parsed.weeks == 0
    assert parsed.remaining_days == 0
    assert parsed.hours == 1
    assert parsed.minutes == 30
    assert parsed.remaining_seconds == 0
    assert parsed.microseconds == 0


def test_parse_duration_no_operator():
    with pytest.raises(ParserError):
        parse("2Y3M4DT5H6M7S")


def test_parse_duration_weeks_combined():
    with pytest.raises(ParserError):
        parse("P1Y2W")


def test_parse_duration_invalid_order():
    with pytest.raises(ParserError):
        parse("P1S")

    with pytest.raises(ParserError):
        parse("P1D1S")

    with pytest.raises(ParserError):
        parse("1Y2M3D1SPT1M")

    with pytest.raises(ParserError):
        parse("P1Y2M3D2MT1S")

    with pytest.raises(ParserError):
        parse("P2M3D1ST1Y1M")

    with pytest.raises(ParserError):
        parse("P1Y2M2MT3D1S")

    with pytest.raises(ParserError):
        parse("P1D1Y1M")

    with pytest.raises(ParserError):
        parse("PT1S1H")


def test_parse_duration_invalid():
    with pytest.raises(ParserError):
        parse("P1Dasdfasdf")


def test_parse_duration_fraction_only_allowed_on_last_component():
    with pytest.raises(ParserError):
        parse("P2Y3M4DT5.5H6M7S")



from __future__ import annotations

import pytest

import pendulum

from tests.conftest import assert_datetime


def test_start_of_second():
    d = pendulum.now()
    new = d.start_of("second")
    assert isinstance(new, pendulum.DateTime)
    assert_datetime(new, d.year, d.month, d.day, d.hour, d.minute, d.second, 0)


def test_end_of_second():
    d = pendulum.now()
    new = d.end_of("second")
    assert isinstance(new, pendulum.DateTime)
    assert_datetime(new, d.year, d.month, d.day, d.hour, d.minute, d.second, 999999)


def test_start_of_minute():
    d = pendulum.now()
    new = d.start_of("minute")
    assert isinstance(new, pendulum.DateTime)
    assert_datetime(new, d.year, d.month, d.day, d.hour, d.minute, 0, 0)


def test_end_of_minute():
    d = pendulum.now()
    new = d.end_of("minute")
    assert isinstance(new, pendulum.DateTime)
    assert_datetime(new, d.year, d.month, d.day, d.hour, d.minute, 59, 999999)


def test_start_of_hour():
    d = pendulum.now()
    new = d.start_of("hour")
    assert isinstance(new, pendulum.DateTime)
    assert_datetime(new, d.year, d.month, d.day, d.hour, 0, 0, 0)


def test_end_of_hour():
    d = pendulum.now()
    new = d.end_of("hour")
    assert isinstance(new, pendulum.DateTime)
    assert_datetime(new, d.year, d.month, d.day, d.hour, 59, 59, 999999)


def test_start_of_day():
    d = pendulum.now()
    new = d.start_of("day")
    assert isinstance(new, pendulum.DateTime)
    assert_datetime(new, d.year, d.month, d.day, 0, 0, 0, 0)


def test_end_of_day():
    d = pendulum.now()
    new = d.end_of("day")
    assert isinstance(new, pendulum.DateTime)
    assert_datetime(new, d.year, d.month, d.day, 23, 59, 59, 999999)


def test_start_of_month_is_fluid():
    d = pendulum.now()
    assert isinstance(d.start_of("month"), pendulum.DateTime)


def test_start_of_month_from_now():
    d = pendulum.now()
    new = d.start_of("month")
    assert_datetime(new, d.year, d.month, 1, 0, 0, 0, 0)


def test_start_of_month_from_last_day():
    d = pendulum.datetime(2000, 1, 31, 2, 3, 4)
    new = d.start_of("month")
    assert_datetime(new, 2000, 1, 1, 0, 0, 0, 0)


def test_start_of_year_is_fluid():
    d = pendulum.now()
    new = d.start_of("year")
    assert isinstance(new, pendulum.DateTime)


def test_start_of_year_from_now():
    d = pendulum.now()
    new = d.start_of("year")
    assert_datetime(new, d.year, 1, 1, 0, 0, 0, 0)


def test_start_of_year_from_first_day():
    d = pendulum.datetime(2000, 1, 1, 1, 1, 1)
    new = d.start_of("year")
    assert_datetime(new, 2000, 1, 1, 0, 0, 0, 0)


def test_start_of_year_from_last_day():
    d = pendulum.datetime(2000, 12, 31, 23, 59, 59)
    new = d.start_of("year")
    assert_datetime(new, 2000, 1, 1, 0, 0, 0, 0)


def test_end_of_month_is_fluid():
    d = pendulum.now()
    assert isinstance(d.end_of("month"), pendulum.DateTime)


def test_end_of_month():
    d = pendulum.datetime(2000, 1, 1, 2, 3, 4).end_of("month")
    new = d.end_of("month")
    assert_datetime(new, 2000, 1, 31, 23, 59, 59)


def test_end_of_month_from_last_day():
    d = pendulum.datetime(2000, 1, 31, 2, 3, 4)
    new = d.end_of("month")
    assert_datetime(new, 2000, 1, 31, 23, 59, 59)


def test_end_of_year_is_fluid():
    d = pendulum.now()
    assert isinstance(d.end_of("year"), pendulum.DateTime)


def test_end_of_year_from_now():
    d = pendulum.now().end_of("year")
    new = d.end_of("year")
    assert_datetime(new, d.year, 12, 31, 23, 59, 59, 999999)


def test_end_of_year_from_first_day():
    d = pendulum.datetime(2000, 1, 1, 1, 1, 1)
    new = d.end_of("year")
    assert_datetime(new, 2000, 12, 31, 23, 59, 59, 999999)


def test_end_of_year_from_last_day():
    d = pendulum.datetime(2000, 12, 31, 23, 59, 59, 999999)
    new = d.end_of("year")
    assert_datetime(new, 2000, 12, 31, 23, 59, 59, 999999)


def test_start_of_decade_is_fluid():
    d = pendulum.now()
    assert isinstance(d.start_of("decade"), pendulum.DateTime)


def test_start_of_decade_from_now():
    d = pendulum.now()
    new = d.start_of("decade")
    assert_datetime(new, d.year - d.year % 10, 1, 1, 0, 0, 0, 0)


def test_start_of_decade_from_first_day():
    d = pendulum.datetime(2000, 1, 1, 1, 1, 1)
    new = d.start_of("decade")
    assert_datetime(new, 2000, 1, 1, 0, 0, 0, 0)


def test_start_of_decade_from_last_day():
    d = pendulum.datetime(2009, 12, 31, 23, 59, 59)
    new = d.start_of("decade")
    assert_datetime(new, 2000, 1, 1, 0, 0, 0, 0)


def test_end_of_decade_is_fluid():
    d = pendulum.now()
    assert isinstance(d.end_of("decade"), pendulum.DateTime)


def test_end_of_decade_from_now():
    d = pendulum.now()
    new = d.end_of("decade")
    assert_datetime(new, d.year - d.year % 10 + 9, 12, 31, 23, 59, 59, 999999)


def test_end_of_decade_from_first_day():
    d = pendulum.datetime(2000, 1, 1, 1, 1, 1)
    new = d.end_of("decade")
    assert_datetime(new, 2009, 12, 31, 23, 59, 59, 999999)


def test_end_of_decade_from_last_day():
    d = pendulum.datetime(2009, 12, 31, 23, 59, 59, 999999)
    new = d.end_of("decade")
    assert_datetime(new, 2009, 12, 31, 23, 59, 59, 999999)


def test_start_of_century_is_fluid():
    d = pendulum.now()
    assert isinstance(d.start_of("century"), pendulum.DateTime)


def test_start_of_century_from_now():
    d = pendulum.now()
    new = d.start_of("century")
    assert_datetime(new, d.year - d.year % 100 + 1, 1, 1, 0, 0, 0, 0)


def test_start_of_century_from_first_day():
    d = pendulum.datetime(2001, 1, 1, 1, 1, 1)
    new = d.start_of("century")
    assert_datetime(new, 2001, 1, 1, 0, 0, 0, 0)


def test_start_of_century_from_last_day():
    d = pendulum.datetime(2100, 12, 31, 23, 59, 59)
    new = d.start_of("century")
    assert_datetime(new, 2001, 1, 1, 0, 0, 0, 0)


def test_end_of_century_is_fluid():
    d = pendulum.now()
    assert isinstance(d.end_of("century"), pendulum.DateTime)


def test_end_of_century_from_now():
    now = pendulum.now()
    d = now.end_of("century")
    assert_datetime(d, now.year - now.year % 100 + 100, 12, 31, 23, 59, 59, 999999)


def test_end_of_century_from_first_day():
    d = pendulum.datetime(2001, 1, 1, 1, 1, 1)
    new = d.end_of("century")
    assert_datetime(new, 2100, 12, 31, 23, 59, 59, 999999)


def test_end_of_century_from_last_day():
    d = pendulum.datetime(2100, 12, 31, 23, 59, 59, 999999)
    new = d.end_of("century")
    assert_datetime(new, 2100, 12, 31, 23, 59, 59, 999999)


def test_average_is_fluid():
    d = pendulum.now().average()
    assert isinstance(d, pendulum.DateTime)


def test_average_from_same():
    d1 = pendulum.datetime(2000, 1, 31, 2, 3, 4)
    d2 = pendulum.datetime(2000, 1, 31, 2, 3, 4).average(d1)
    assert_datetime(d2, 2000, 1, 31, 2, 3, 4)


def test_average_from_greater():
    d1 = pendulum.datetime(2000, 1, 1, 1, 1, 1, tz="local")
    d2 = pendulum.datetime(2009, 12, 31, 23, 59, 59, tz="local").average(d1)
    assert_datetime(d2, 2004, 12, 31, 12, 30, 30)


def test_average_from_lower():
    d1 = pendulum.datetime(2009, 12, 31, 23, 59, 59, tz="local")
    d2 = pendulum.datetime(2000, 1, 1, 1, 1, 1, tz="local").average(d1)
    assert_datetime(d2, 2004, 12, 31, 12, 30, 30)


def start_of_with_invalid_unit():
    with pytest.raises(ValueError):
        pendulum.now().start_of("invalid")


def end_of_with_invalid_unit():
    with pytest.raises(ValueError):
        pendulum.now().end_of("invalid")


def test_start_of_with_transition():
    d = pendulum.datetime(2013, 10, 27, 23, 59, 59, tz="Europe/Paris")
    assert d.offset == 3600
    assert d.start_of("month").offset == 7200
    assert d.start_of("day").offset == 7200
    assert d.start_of("year").offset == 3600


def test_start_of_on_date_before_transition():
    d = pendulum.datetime(2013, 10, 27, 0, 59, 59, tz="UTC").in_timezone("Europe/Paris")
    assert d.offset == 7200
    assert d.start_of("minute").offset == 7200
    assert d.start_of("hour").offset == 7200
    assert d.start_of("day").offset == 7200
    assert d.start_of("month").offset == 7200
    assert d.start_of("year").offset == 3600


def test_start_of_on_date_after_transition():
    d = pendulum.datetime(2013, 10, 27, 1, 59, 59, tz="UTC").in_timezone("Europe/Paris")
    assert d.offset == 3600
    assert d.start_of("minute").offset == 3600
    assert d.start_of("hour").offset == 3600
    assert d.start_of("day").offset == 7200
    assert d.start_of("month").offset == 7200
    assert d.start_of("year").offset == 3600


def test_end_of_with_transition():
    d = pendulum.datetime(2013, 3, 31, tz="Europe/Paris")
    assert d.offset == 3600
    assert d.end_of("month").offset == 7200
    assert d.end_of("day").offset == 7200
    assert d.end_of("year").offset == 3600


def test_end_of_on_date_before_transition():
    d = pendulum.datetime(2013, 10, 27, 0, 0, 0, tz="UTC").in_timezone("Europe/Paris")
    assert d.offset == 7200
    assert d.end_of("minute").offset == 7200
    assert d.end_of("hour").offset == 7200
    assert d.end_of("day").offset == 3600
    assert d.end_of("month").offset == 3600
    assert d.end_of("year").offset == 3600


def test_end_of_on_date_after_transition():
    d = pendulum.datetime(2013, 10, 27, 1, 0, 0, tz="UTC").in_timezone("Europe/Paris")
    assert d.offset == 3600
    assert d.end_of("minute").offset == 3600
    assert d.end_of("hour").offset == 3600
    assert d.end_of("day").offset == 3600
    assert d.end_of("month").offset == 3600
    assert d.end_of("year").offset == 3600



from __future__ import annotations

import re

from importlib import import_module, resources
from pathlib import Path
from typing import Any
from typing import ClassVar
from typing import Dict
from typing import cast


class Locale:
    """
    Represent a specific locale.
    """

    _cache: ClassVar[dict[str, Locale]] = {}

    def __init__(self, locale: str, data: Any) -> None:
        self._locale: str = locale
        self._data: Any = data
        self._key_cache: dict[str, str] = {}

    @classmethod
    def load(cls, locale: str | Locale) -> Locale:
        if isinstance(locale, Locale):
            return locale

        locale = cls.normalize_locale(locale)
        if locale in cls._cache:
            return cls._cache[locale]

        # Checking locale existence
        actual_locale = locale
        locale_path = cast(Path, resources.files(__package__).joinpath(actual_locale))
        while not locale_path.exists():
            if actual_locale == locale:
                raise ValueError(f"Locale [{locale}] does not exist.")

            actual_locale = actual_locale.split("_")[0]

        m = import_module(f"pendulum.locales.{actual_locale}.locale")

        cls._cache[locale] = cls(locale, m.locale)

        return cls._cache[locale]

    @classmethod
    def normalize_locale(cls, locale: str) -> str:
        m = re.match("([a-z]{2})[-_]([a-z]{2})", locale, re.I)
        if m:
            return f"{m.group(1).lower()}_{m.group(2).lower()}"
        else:
            return locale.lower()

    def get(self, key: str, default: Any | None = None) -> Any:
        if key in self._key_cache:
            return self._key_cache[key]

        parts = key.split(".")
        try:
            result = self._data[parts[0]]
            for part in parts[1:]:
                result = result[part]
        except KeyError:
            result = default

        self._key_cache[key] = result

        return self._key_cache[key]

    def translation(self, key: str) -> Any:
        return self.get(f"translations.{key}")

    def plural(self, number: int) -> str:
        return cast(str, self._data["plural"](number))

    def ordinal(self, number: int) -> str:
        return cast(str, self._data["ordinal"](number))

    def ordinalize(self, number: int) -> str:
        ordinal = self.get(f"custom.ordinal.{self.ordinal(number)}")

        if not ordinal:
            return f"{number}"

        return f"{number}{ordinal}"

    def match_translation(self, key: str, value: Any) -> dict[str, str] | None:
        translations = self.translation(key)
        if value not in translations.values():
            return None

        return cast(Dict[str, str], {v: k for k, v in translations.items()}[value])

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}('{self._locale}')"



from __future__ import annotations

import pendulum


locale = "nl"


def test_diff_for_humans():
    with pendulum.travel_to(pendulum.datetime(2016, 8, 29), freeze=True):
        diff_for_humans()


def diff_for_humans():
    d = pendulum.now().subtract(seconds=1)
    assert d.diff_for_humans(locale=locale) == "enkele seconden geleden"

    d = pendulum.now().subtract(seconds=2)
    assert d.diff_for_humans(locale=locale) == "enkele seconden geleden"

    d = pendulum.now().subtract(seconds=22)
    assert d.diff_for_humans(locale=locale) == "22 seconden geleden"

    d = pendulum.now().subtract(minutes=1)
    assert d.diff_for_humans(locale=locale) == "1 minuut geleden"

    d = pendulum.now().subtract(minutes=2)
    assert d.diff_for_humans(locale=locale) == "2 minuten geleden"

    d = pendulum.now().subtract(hours=1)
    assert d.diff_for_humans(locale=locale) == "1 uur geleden"

    d = pendulum.now().subtract(hours=2)
    assert d.diff_for_humans(locale=locale) == "2 uur geleden"

    d = pendulum.now().subtract(days=1)
    assert d.diff_for_humans(locale=locale) == "1 dag geleden"

    d = pendulum.now().subtract(days=2)
    assert d.diff_for_humans(locale=locale) == "2 dagen geleden"

    d = pendulum.now().subtract(weeks=1)
    assert d.diff_for_humans(locale=locale) == "1 week geleden"

    d = pendulum.now().subtract(weeks=2)
    assert d.diff_for_humans(locale=locale) == "2 weken geleden"

    d = pendulum.now().subtract(months=1)
    assert d.diff_for_humans(locale=locale) == "1 maand geleden"

    d = pendulum.now().subtract(months=2)
    assert d.diff_for_humans(locale=locale) == "2 maanden geleden"

    d = pendulum.now().subtract(years=1)
    assert d.diff_for_humans(locale=locale) == "1 jaar geleden"

    d = pendulum.now().subtract(years=2)
    assert d.diff_for_humans(locale=locale) == "2 jaar geleden"

    d = pendulum.now().add(seconds=1)
    assert d.diff_for_humans(locale=locale) == "over enkele seconden"

    d = pendulum.now().add(weeks=1)
    assert d.diff_for_humans(locale=locale) == "over 1 week"

    d = pendulum.now().add(seconds=1)
    d2 = pendulum.now()
    assert d.diff_for_humans(d2, locale=locale) == "enkele seconden later"
    assert d2.diff_for_humans(d, locale=locale) == "enkele seconden eerder"

    assert d.diff_for_humans(d2, True, locale=locale) == "enkele seconden"
    assert (
        d2.diff_for_humans(d.add(seconds=1), True, locale=locale) == "enkele seconden"
    )


def test_format():
    d = pendulum.datetime(2016, 8, 28, 7, 3, 6, 123456)
    assert d.format("dddd", locale=locale) == "zondag"
    assert d.format("ddd", locale=locale) == "zo"
    assert d.format("MMMM", locale=locale) == "augustus"
    assert d.format("MMM", locale=locale) == "aug."
    assert d.format("A", locale=locale) == "a.m."
    assert d.format("Do", locale=locale) == "28e"
